/*
	NRob Library
		CPU　ATMEGA328P
			motor Driver:TB6612 D2-D7
			Servo	D8,D9
			超音波センサ	D10-D13
			Analog 6ケ　A0
		I2C Expander MVP23017
			LCD 16*2
			SW 5ケ
v1.0	20210410	ライブラリ化
v1.1	20211106	loop動作見直し
//v363	20230114	SW入力　センサーS0に追加

*/


#ifndef NROB_h
#define NROB_h

#include <Wire.h>
#include <Arduino.h>
#include <EEPROM.h>
#include <Servo.h>


//TB6612  motor Driver
#define MTA1    2       //AIN1
#define MTA2    3       //AIN2
#define MTAP    5       //PWMA
#define MTB1    4       //BIN1
#define MTB2    7       //BIN2
#define MTBP    6       //PWMB

//Servo
#define SVPIN_M3  9 
#define SVPIN_M4  8 
//#define SV_SPEED	10
//#define SV_WAIT		1		//wait for servo movinng

//Ultrasonic
#define S7TRIG  10
#define S7ECHO  11
#define S8TRIG  12
#define S8ECHO  13

//コンパス
//  GY271                     //コンパスセンサー
#define ADR271    0x0D        //GY271アドレス

//  GY9250
#define ADR9250   0x68        //MPU9250アドレス
#define ADR8963   0x0C        //磁気センサのスレーブアドレス

//	MCP23017　IOエクスパンダ
#define MCP23017_ADR  0x20    //I2C　アドレス
#define SW_LONG   100         //SW長押し判定　(x10ms)

//LCD Drive Port
#define BIT_RS 0x40            
#define BIT_RW 0x20
#define BIT_EN 0x10
#define BIT_D4 0x08
#define BIT_D5 0x04
#define BIT_D6 0x02
#define BIT_D7 0x01

//サブボード
#define ADR_SUB_IN	0x18			//入力用サブボード
#define ADR_SUB_OUT	0x19			//出力用サブボード
#define COM_SUB_IN	0x81			//入力コマンド
#define COM_SUB_OUT	0x16			//出力コマンド
#define SUB_NUMBER	0x0A			//サブポート数


//----------------------------------------------------------
class NRob3												
{
  public:

		void	init(int pich);								//初期化(loopでオンするピッチ(ms))

		bool	loop(void);										//基本ループ 50ms毎true

		int  	get_AD(byte no);							//AD測定値を返す
		bool	in_SW(byte no);
		bool	in_LongSW(byte no);
		bool	key_SW(byte no);							//SWの状態確認　v363

		void	act_motorAB(int pa, int pb);	//
		void	act_motorA(int pa);						//
		void	act_motorB(int pb);						//

		void	act_ServoDir(int i, int p);		//サーボ値指定駆動 i=servoNo(1or2)
		void	act_ServoAbs(int i, int p);		//サーボ絶対値駆動 i=servoNo(1or2)
		void	servo_ON(void);        	      //
		void	servo_OFF(void);
		int		get_ServoPos(int no);					//サーボの現在位置を返す

		int		get_US(int no);								//超音波センサ測定値読込
		void	use_US(byte num);							//超音波センサ使用 1=1ケ/2=2ケ/0=無し

		void	use_SubIn(byte num);					//入力子機使用 1=あり/0=無し
		int		get_SubIn(byte n);						//入力子機値を返す
		void	write_SubOut(byte no, int d);	//出力子機に書き出し

		bool  chk_Compass(void);						//コンパスの接続を確認
		void	init_Compass(void);						//コンパスモジュール初期化
		void	read_Compass(void);						//コンパス読み込み動作
		int		get_Angle(void);							//コンパス読み込み
		int		get_RelAng(void);							//相対角度読み込み
		void  get_OffsetCompas(void);				//コンパスオフセット調整 v351

		void	LED_ON(byte no);
		void	LED_OFF(byte no);
		void	LED_AllOFF();

		void  LCD_clear(void);
		void  LCD_print(char *w);
		void  LCD_printF(char *w);
		void  LCD_write(byte w);
		void  LCD_setCursor(byte x, byte y);
		void  LCD_noCursor(void);
		void  LCD_cursor(void);
		void  LCD_noBlink(void);
		void  LCD_blink(void);

		void  dsp_Number2(byte x, byte y, int no);	//２桁数字表示
		void  dsp_NumberB(byte x, byte y, byte n);	//1バイト表示(3桁)
		void  dsp_Number4(byte x, byte y, int n);		//４桁数字表示

		byte  I2C_Check(byte ad);
		void	I2C_Send(byte num);						//I2C 送信　num:送信バイト数
		byte	I2C_Recv(byte num);						//I2C 受信　num:受信バイト数
		byte	I2C_Recv0(byte num);					//I2C 受信のみ

		byte	parK[10];											//EEPメモリ定数

		int		ax, ay, az;										//MPU9270 加速度
		int		gx, gy, gz;										//MPU9270	ジャイロ
		int		mx, my, mz;										//コンパスセンサ　各軸

		bool	flgServoAct=false;						//サーボ動作フラグ

	private:

		void  Special_Mode(void);						//特殊モード
		void  dsp_ADmoni(void);							//アナログモニタ表示
		void	dsp_USmoni(void);							//超音波モニタ表示
		void  dsp_Compasmoni(void);					//コンパスモニタ表示
		void  write_ofset(void);						//オフセットメモリ v351
		void  read_ofset(void);							//オフセットメモリ v351
		void  dsp_sINmoni(void);						//カラーセンサモニタ表示
		void  set_ParaK(void);							//パラメーター設定画面
		void  dsp_parK(byte no);						//Kパラメーター表示
		void	read_parK(void);							//Kパラメーター読込
		void	write_parK(void);							//Kパラメーター書込み

		void	init_Tm2(void);								//Timer2動作設定

		void	chk_SW(void);									//Every 10second
		bool  chk_AD(void);              		//ADを順に取込
		void	chk_US(void);									//超音波センサ読込動作

		void	init_271(void);								//GY271初期化
		void	read_271(void);								//コンパス読み込み
		void  init_9250(void);							//コンパス9250初期化
		void  read_9250(void);							//コンパス9250読み込み
		void	chg360(int x, int y);					//コンパス360度換算 共用部

		void	read_SubIn(void);							//入力子機カラーセンサ読み込み

		void	LCD_init(void);
		void  LCD_Send4(byte value, byte mode);
		void	LED_send();
		void	act_ServoLoop(void);					//サーボloop動作

		int		usrTime;											//指定のピッチ(loopでピッチを返す）
		int		cntUsr;												// ↑　のカウンタ

		byte	isSonar=0;										//超音波を使っているか？
		byte	isSubIn=0;										//入力子機を使っているか？

		int	mpowA, mpowB;
		Servo svM3;
		Servo svM4;

		byte	inSW[5];    //SW input state 0:no/1:in/2:wait/3:longwait
		byte	tmSW[5];
		byte	outGPA;     //GPAsend
		byte	outGPA0;		//前回のGPA
		//byte	outGPB;     //GPB
		byte	inGPAnow;   //GPAin now
		byte	inGPApre;   //GPAin before
		byte	displayOn;

		int   adPin[6]={0,1,2,3,6,7};
		int   adCnt=0;

		int		ave[6];												//AD平均値用
		int		adAdd[6];
		int		aveCnt=0;

		int		inVar[10];										//通信入力変数 
	
		int		us[2];												//超音波
		int   svPos[2]={180,180};   				//サーボポジション
		int		svTag[2]={180,180};						//サーボ移動先

		byte	typeCompass=0;								//コンパスの種類
		int		angAbs;												//360°換算角度
		int		angOrg;												//スタート時の角度
		int		angRel;												//スタート時相対角度　スタート時50
		int		xOfset,yOfset,zOfset;					//コンパスオフセット

		byte	I2Cadr;
		byte	I2Cbuf[24];

};


#endif

/*
//  ********************************************************
NRob3クラスリファレンス

Atmega328 Arduino pro mini 互換
//---ポート設定----
//D00   RX
//D01   TX
//D02   TB6612  INA1
//D03   TB6612  INA2
//D04   TB6612  INB1
//D05   TB6612  PWMA
//D06   TB6612  PWMB
//D07   TB6612  INB2
//D08   Servo M4 SIG
//D09   Servo M3 SIG
//D10   Sonar S7 Trig
//D11   Sonar S7 Echo
//D12   Sonar S8 Trig
//D13   Sonar S8 Echo
//A0    AD input S1
//A1    AD input S2
//A2    AD input S3
//A3    AD input S4
//A4    I2C SDA
//A5    I2C SCL
//A6    AD input S5
//A7    AD input S6

//MCP23017　　I2C拡張
//GPA0  SWS Select
//GPA1  SWD Down
//GPA2  SWU Up
//GPA3  SWR Right
//GPA4  SWL Left
//GPA5  LED3
//GPA6  LED2
//GPA7  LED1
//GPB0  LCD1602 LD7
//GPB1  LCD1602 LD6
//GPB2  LCD1602 LD5
//GPB3  LCD1602 LD4
//GPB4  LCD1602 LE
//GPB5  LCD1602 LRW
//GPB6  LCD1602 LRS
//GPB7  x
//-------------------------------------

NRob.hをインクルード
注)　サーボ駆動に <VarSpeedServo.h> が必要
タイマ２割り込み利用　2ms 割込み内でカウントし10ms毎の動作を行う
実行のループ内で loopをコールする必要あり

※　センサモニタ動作
SelSWを押しながら電源ONでセンサモニタモードに入る
 -SelSWでページ切替

1ページ：上S1,S2,S3/下S4,S5,S6 の入力値モニタ
2ページ：超音波センサ　US1,US2 の入力値モニタ
3ページ：I1　入力子機値　上i0,i1,i2 / 下i3,i4
4ページ：I2　入力子機値　上i5,i6,i7 / 下i8,i9
5ページ：コンパス　上 a=方角絶対値 / 下 r=電源ONの方向からの相対値
6ページ：調整用パラメータ設定画面
　EEPROMに10ケのパラメータがセットできるそのparKをセットするページ
　上下SWで番号選び　1桁ごとSelSWで選択して数字を変えることが出来る

※コンパスモニタ時にUpSWで、オフセット調整に入る。ここでコンパスを一周回してから
、DownSWを押すとオフセット値を書き換えることができる。


//----------------------------------------------------------
void	init(int pich);			初期化 
	引:loopでオンするピッチ(ms)
	戻:　
　：周辺機器の初期化等　setupルーチンからコール
		I2C初期化、pinmode、MPC23017初期化、LCD初期化、コンパス初期化
		タイマ初期化
//----------------------------------------------------------
bool	loop(void);									基本ループ 
	引:
	戻:指定ピッチ毎trueを返す　
	：毎回AD動作、
		10msフラグでSWのチェック、
		10msを指定ピッチカウントでtrueを返す。
		-超音波,コンパス,入力子機　読込み

//  ********************************************************
//------
int  	get_AD(byte no);						AD測定値を返す
	引:AD値番号　1～6／S1～S2コネクタ入力値　5回平均値
	戻:AD入力平均値を返す
	：loop内でchk_ADで検出したAD値を平均　その平均値を返す

//------
bool	in_SW(byte no)					SWの状態を返す
	引:no=SW番号　0=Select/1=Down/2=Up/3=Right/4=Left
	戻:true=SW受付 /false=以外(押されてない、離し待ちなど)
	：chk_SWで検出したSWの状態を返す

//------
bool	in_LongSW(byte no)			長押し確認
	引:no=SW番号　0=Select/1=Down/2=Up/3=Right/4=Left
	戻:true=SW長押し受付 /false=以外

//------
bool	key_SW(byte no)				SW状態確認
	引:no=SW番号　0=Select/1=Down/2=Up/3=Right/4=Left
	戻:true=押されている /false=以外

//  ********************************************************
//------
void	act_motorAB(int pa, int pb)	モータの駆動　AB両方
	引:pa=M1モーター/pb=M2モーター　パワー
	戻:指定ピッチ毎trueを返す　
	：TB6612を駆動　act_moterA、act_moterBをコール

//------
void	act_motorA(int pa)
void	act_motorB(int pb)
	：TB6612駆動　変動なければ動作継続
	：pa,pb -255～+255

//  ********************************************************
//----------------------------------------------------------
void	act_ServoRel(int i, int p) //サーボ相対駆動
	引: i:サーボ番号=(1or2)、p:相対位置　+-180
	戻:
	：現在位置からの相対角度で指令

//------
void	act_ServoAbs(int i, int p)	//サーボ絶対値駆動
	引: i:サーボ番号=(1or2)、p:絶対位置　0～180
	戻:
	：絶対位置でサーボの角度指令

//------
void	servo_ON(void)       サーボON/OFF
void	servo_OFF(void) 
	引:	戻:
	：サーボリギング対策でON/OFFする場合使用

//------
int  get_ServoPos(int no)	//サーボの現在位置を返す
	引:no:サーボ番号=(1or2)
	戻:角度を返す
	：絶対位置でサーボの角度位置を返す

//  ********************************************************
//----------------------------------------------------------
int		get_US(int no)
	引:no:超音波センサ番号=(1or2)
	戻:距離を返すcm
	：規定時間毎超音波センサ読込中　その値を返す

//------
void	use_US(byte num) 			//超音波センサ使用 1=1ケ/2=2ケ/0=無し
	引:超音波センサフラグ
	戻:
	：規定時間毎超音波センサ読込中　その値を返す

//  ********************************************************
//----------------------------------------------------------
void	use_SubIn(byte num)					//入力子機使用 1=あり/0=無し
	引:入力子機使用するか 1=あり/0=無し
	戻:
	：入力子機のフラグを立てる　loopで一定ピッチで取込変数にセット

//------
int		get_SubIn(byte n)						//入力子機値を返す
	引:変数番号　0-9
	戻:
	：入力された値

//------
void	write_SubOut(byte no, int d)	//出力子機に書き出し
	引:no=変数番号 0-9 / d=出力値
	戻:

//  ********************************************************
//----------------------------------------------------------
bool  chk_Compass(void)				コンパスの接続を確認
	引:
	戻:true=コンパスあり/false=コンパスなし
	：I2Cのコンパスセンサが接続されているかをチェック

//------
void	init_Compass(void)			コンパスモジュール初期化
	引:
	戻:
	：コンパスセンサの初期化、gy271又gy9250どちらか接続されているセンサー
		を初期化、相対角度算出用に最初の方向を読み込み

//------
void	read_Compass(void)			コンパス読み込み動作
	引:
	戻:
	：コンパスの読込み動作、読み込んで絶対角度、相対角度を更新

//------
int		get_Angle(void)					コンパス読み込み
	引:
	戻:読み込んでいる絶対角度を返す(0～360)

//------
int		get_RelAng(void)				相対角度読み込み
	引:
	戻:読み込んでいる相対角度を返す(0～360)

//  ********************************************************
//----------------------------------------------------------
void	LED_ON(byte no)					LED ON
	引:no=LED番号(1～3)
	戻:

//------
void	LED_OFF(byte no)        LED OFF
	引:no=LED番号(1～3)
	戻:

//------
void	LED_AllOFF()            LED 全消灯
	引:
	戻:

//  ********************************************************
//----------------------------------------------------------
void  LCD_clear(void)					LCD全消し
	引:
	戻:

//------
void  LCD_print(char *w)			現在のカーソルから文字列表示
	引:表示する文字列
	戻:

//------
void  LCD_write(byte w)				現在のカーソルから１文字表示
	引:表示する文字
	戻:

//------
void  LCD_setCursor(byte x,byte y)	カーソル位置の移動
	引:移動する位置　x(ヨコ):0-15,y(タテ):0-1
	戻:

//------
void  LCD_noCursor(void)			カーソル表示を消す
	引:
	戻:

//------
void  LCD_cursor(void)				カーソルを表示する
	引:
	戻:

//------
void  LCD_noBlink(void)				カーソルブリンクを止める
	引:
	戻:

//------
void  LCD_blink(void)					カーソルブリンクをする
	引:
	戻:

//  ********************************************************
//----------------------------------------------------------
void  dsp_Number2(byte x, byte y, int no);	//２桁数字表示
	引:x,y位置にnoの数字を2けた表示
	戻:

//------
void  dsp_NumberB(byte x, byte y, byte n);	//1バイト表示(3桁)
	引:x,y位置にnoの数字を3けた表示
	戻:

//------
void  dsp_Number4(byte x, byte y, int n);		//４桁数字表示
	引:x,y位置にnoの数字を4けた表示
	戻:

//  ********************************************************
//----------------------------------------------------------
byte  I2C_Check(byte ad)			I2C子機が接続されているかチェック
	引:テストするアドレス
	戻:OKなら0を返す

void	I2C_Send(byte num)			I2C 送信　num:送信バイト数
	引:送信バイト数
	戻:

byte	I2C_Recv(byte num);			I2C 受信　num:受信バイト数
	引:受信バイト数
	戻:受信バイト数

byte	I2C_Recv0(byte num);					//I2C 受信のみ
	引:受信バイト数
	戻:受信バイト数

//  ********************************************************
byte	parK[10];											//EEPメモリ定数
　動作調整用パラメータ変数　1バイト10ケ

//  ********************************************************
		int		ax, ay, az;										//MPU9270 加速度
		int		gx, gy, gz;										//MPU9270	ジャイロ
		int		mx, my, mz;										//コンパスセンサ　各軸
MPU9250を使った場合、加速度とジャイロを入力だけ行いメモリする。

//  ********************************************************
--private	----------------------------------------

void  Special_Mode(void);						//特殊モード
	-		initよりコール　センサモニタ動作 以下その関連

	void  dsp_ADmoni(void);							//アナログモニタ表示
	void	dsp_USmoni(void);							//超音波モニタ表示
	void  dsp_Compasmoni(void);					//コンパスモニタ表示
	void  dsp_sINmoni(void);						//カラーセンサモニタ表示
	void  set_ParaK(void);							//パラメーター設定画面
	void  dsp_parK(byte no);						//Kパラメーター表示
	void	read_parK(void);							//Kパラメーター読込
	void	write_parK(void);							//Kパラメーター書込み

void	init_Tm2(void);								//Timer2動作設定
	-		initよりコール　タイマ割込みのセット

void	chk_SW(void);									//Every 10second
	-	loopよりコール　SWの入力を見る

bool  chk_AD(void);              		//ADを順に取込
	-	loopよりコール　アナログを順に入力

void	chk_US(void);									//超音波センサ読込動作
	-	loopよりコール　超音波センサを定期的に読込

-以下コンパス　GY271とMPU9250の操作
	void	init_271(void);								//GY271初期化
	void	read_271(void);								//コンパス読み込み
	void  init_9250(void);							//コンパス9250初期化
	void  read_9250(void);							//コンパス9250読み込み


void	read_SubIn(void);							//入力子機カラーセンサ読み込み
	-	loopよりコール　入力子機の値を定期的に読込

-MCP23017　接続LED,LCDへの出力関係
	void	LCD_init(void);
	void  LCD_Send4(byte value, byte mode);
	void	LED_send();

--変数	----------------------------------------
	int		usrTime;											//指定のピッチ(loopでピッチを返す）
	int		cntUsr;												// ↑　のカウンタ

	byte	isSonar=0;										//超音波を使っているか？
	byte	isSubIn=0;										//入力子機を使っているか？

	int	mpowA, mpowB;
	Servo svM3;
	Servo svM4;

	byte	inSW[5];    //SW input state 0:no/1:in/2:wait/3:longwait
	byte	tmSW[5];
	byte	outGPA;     //GPAsend
	byte	outGPB;     //GPB
	byte	inGPAnow;   //GPAin now
	byte	inGPApre;   //GPAin before
	byte	displayOn;

	int   adPin[6]={0,1,2,3,6,7};
	int   adCnt=0;
	int		ave[6];												//AD平均値用
	int		adAdd[6];
	int		aveCnt=0;

	int		inVar[10];										//通信入力変数 
	
	int		us[2];												//超音波
	int   svPos[2]={50,50};   					//サーボポジション

	byte	typeCompass=0;								//コンパスの種類
	int		angAbs;												//360°換算角度
	int		angOrg;												//スタート時の角度
	int		angRel;												//スタート時相対角度　スタート時50
	int		xOfset,yOfset,zOfset;					//コンパスオフセット

	byte	I2Cadr;												//I2Cアドレス
	byte	I2Cbuf[24];										//I2Cバッファ

*/

