
//NRob3 プログラム作成
//v20  20190812  NRob32基板に対応
//v23 20190818  TB6612駆動方法変更
//v311  20191002  低学年用簡易ソース 実行動作区別
//v312  20191013  動作部修正
//v312b  20191018  サーボ必要時のみ駆動
//v321  20191028  NRob33基板　旧メモリタイプでソース整理
//v322  20191101  v321の簡易版変数等なし
//v331  20191117  メモリ簡略版テスト
//v332  20200214  I2C通信エラー対策
//v333  20200301  S番号違い修正
//v334  20200303  B、L命令変更
//v335  20200308  ディバッグテスト用
//v336a 20200323  ライブラリ分割 TB6612,sevo,ultrsonic,LED
//v337	20200601	ライブラリ分割
//v338	20200627	InsDel修正
//v339	20200711	Com_PC 修正
//v340	20200721	ジャンプ命令修正
//v341	20200824	ジャンプ命令再修正
//v342	20200915	$命令登録ミス修正
//v343	20201022	waitセンサーチェックする
//v344	20201023	AD　NRobクラスに統一　＋平均動作 サブルーチン内のジャンプ
//v345	20201028	コンパス　271　9250共用
//v346	20201106	$の形式符号なしに変更
//v347	20201118	タイマ割込みに
//v348	20201201	kパラメータeepにメモリ
//v349	20201217	betwinテスト
//v350	20210122	カラーセンサ組み込み　Iを入れる
//v351	20210306	出力ボード　ＳＷを入れる 
//v352	20210412	コンパスキャリブレーション追加
//v353	20210413	ライブラリ化
//v354	20210504	判定行B外しのバグ修正
//v355	20210604	超音波センサ　動作しないバグ
//v356	20210717	JB入力,Bジャンプできないバグ,10行スクロール追加
//v357	20211108	Nrob loopルーチン見直し対応 サーボ動作修正
//v358	20220109	通信動作修正　メニューで通信開始　書き込みは返信
//v359	20221012	計算データーをC4-9に入れる
//v360	20221019	代入命令見直し
//v361	20221207	ループ命令の追加 JLQ,JLS,JLB,JLE
//v362	20221220	NRobDspライブラリ統合
//v363	20230114	SW入力　センサーS0に追加

//---ポート設定----
//D00   RX
//D01   TX
//D02   TB6612  INA1
//D03   TB6612  INA2
//D04   TB6612  INB1
//D05   TB6612  PWMA
//D06   TB6612  PWMB
//D07   TB6612  INB2
//D08   Servo M3 SIG
//D09   Servo M4 SIG
//D10   Sonar S7 Trig
//D11   Sonar S7 Echo
//D12   Sonar S8 Trig
//D13   Sonar S8 Echo
//A0    AD input S1
//A1    AD input S2
//A2    AD input S3
//A3    AD input S4
//A4    I2C SDA
//A5    I2C SCL
//A6    AD input S5
//A7    AD input S6

//MCP23017
//GPA0  SWS Select
//GPA1  SWD Down
//GPA2  SWU Up
//GPA3  SWR Right
//GPA4  SWL Left
//GPA5  LED3
//GPA6  LED2
//GPA7  LED1
//GPB0  LCD1602 LD7
//GPB1  LCD1602 LD6
//GPB2  LCD1602 LD5
//GPB3  LCD1602 LD4
//GPB4  LCD1602 LE
//GPB5  LCD1602 LRW
//GPB6  LCD1602 LRS
//GPB7  x
//-------------------------------------


/*
*データ構造
  1 階層　正負
    上位4ビット：階層0-4/下位４ビット 各階層の正負(YorN)
  2 コマンド  下表
  3-5 引数  ：　0-99:正整数/100-199：負整数/200-209：S1-9/210-219:C1-9/
              220-229:X1-9/230-239:T1-9/240-249:記号

//コマンド
//01  G motor+wait +00,+00,00(LR+wait)
//02  M motor +00,+00(LR)
//03  V servo no,00
//04  W wait  00
//05  L LED,LCD no,+00        //L4=+50
//06	O	I2C Output no,0orX		//O1:+30

//10  ? if S  no,kigo,00
//11  ? if C  
//12  ? if X
//13  ? if T
//14  ? if R  rimocom

//20  $ betwen S  no,lo,hi
//21  $ betwen C  no,lo,hi
//22  $ betwen X  no,lo,hi
//23  $ betwen T  no,lo,hi

//廃止　32  # set X no,kigo,0orX　v360
//33  # set T no,kigo,0orX
//止め　34  # set Lcd no,Xno
//止め  35  # set B 0
//---------------------------

//40  B:  blockNo   例:B:01  /mem:0,40,no,0,0
//41  JB: blockNo   例:JB:01 /mem:X,41,no,0,0
//42  JN: lineNo    例:JN:12 /mem:X,42,no,0,0

//43	JLQ:□,□,□ ループの開始　例:JLQ:X1<03  /mem:0,43,0orX,kigo,0orX v361
//44	JLB:□,□,□ ループの中断　例:JLB:X1<03  /mem:0,45,0orX,kigo,0orX
//45	JLS:---			 ループの開始無条件	例:JLS:  /mem:0,44,0,0,0
//46	JLE:---			 ループの終わり		例:JLE:  	 /mem:0,46,0,0,0


//50-59	# set X(no) 0orX,kigo,0orX	計算代入とする	v360

//S1-6：AD入力コネクタ 
//S7,8:超音波センサー入力　00～99cm
//S9:SW入力チェック　押されたときに１回ON　1=Select/2=Down/3=Up/4=Right/5=Left　v363
//S0:SW状態チェック 押されていると番号 1=Select/2=Down/3=Up/4=Right/5=Left
//C1:コンパス
//C3-8:S1-4の計算値
//S1とS2で計算　C3:比率0-10/C4:大きい値/C5:小さい値
//S3とS4で計算　C6:比率0-10/C7:大きい値/C8:小さい値
//C0:乱数0-99
                   //0   1   2   3   4   5   6   7   8   9   0   1 , 2
command[COMNum+1]={ '+','-','G','M','V','L','O','W','?','$','#','B','J'};

//---- テーブル形式---
unsigned char prgMem[MAXLINE][5];
                            // 01234567890123
unsigned char dspPrg[2][14]={{"G+50+50;10    "},
                             {"B=02          "}};
                        //0   1   2   3   4   5   6   7   8   9
unsigned char SKIGO[]=  {'=','<','>','!','+','-','*','/',' ',' '};
unsigned char VARI[]=   {'S','C','X','T','L','I','O','B',' ',' '};//v350

/* 関数リスト
//*** メイン  ***********************************************
setup:
                    init_MPC23017/LCD_init/init_TB6612/init_PrgData
loop:
    モード切替
    0=chg_Mode_main();        //モード切替動作
    1=do_Action_main();       //プログラム実行動作
    2=do_Setting_main();      //センサーモニタ
    3=
    4=edit_Prg_main();        //エディタ動作

void	timer_Int(void)										//基準時間カウント タイマ割込の代り
void	timer_getSW(void)									//タイマ動作＋SW入力
byte	is_SelSW(void)										//SelSW入力+タイマ動作

chg_Mode_main()               //モード変更動作
                              初期表示
                              Up/Downキー入力でメニュー切替 PAGENUM=4枚
                              selキーで確定

//*** 初期化  ***********************************************
void	init_PrgData(void)								//プログラムデータ初期化

//*** モード変更動作  ***************************************
void	chg_Mode_main(void)								//モード変更動作メイン

//*** 動作  *************************************************
void	do_Action_main(void)							//プログラム実行動作メイン
void	prg_Analyse(void)									//プログラム解析
byte	ctrl_Branch(void)									//条件分岐  ans:1=実行/2=スキップ
void	do_command(void)									//コマンド動作
void	do_MoveCom(void)									//コマンド 動作系
void	do_Go(void)												//Goコマンド モーター＋wait
void	do_Motor(void)										//モーターコマンド
void	do_Servo(void)										//サーボ
void	do_Wait(void)											//Waitコマンド
void	do_IfCom(void)										//? 判定動作
void	do_WithinCom(void)								//$ 範囲判定動作
void	do_SetX(void)											//# 変数セット命令 X
void	do_SetTim(void)										//# 変数セット命令 T
void	do_LEDLCD(void)										//L LED,LCD操作
void	dsp_L_All(void)										//L変数表示
void	do_JumpCom(void)									//J 動作
void	do_return(void)										//Block終わりでreturn
void	do_jumpB(void)										//Blockジャンプ(サブルーチンジャンプ)
int		seek_Block(byte no)								//Block を探す
void	do_jumpN(void)										//番号へ直接ジャンプ
void	do_loopQ(void)										//条件付きループ	v361
void	do_loopB(void)										//ループ飛び出し	v361
byte	judg_234(int no)									//パラメータ２，３，４の判定	v361
void	do_loopS(void)										//無条件ループ開始	v361
void	do_loopE(void)										//ループの終わり	v361
int	seek_loopTop(void)									//ループの始まりを探す	v361
int	seek_loopEnd(void)									//ループの終わりを探す	v361
int		chg_Argument(byte n)							//引数を数字に変換 正負＋変数
void	Timer_Count(void)									//タイマーカウント

//*** AD入力    *********************************************
void	get_AD(void)											//ADを順に取込

//*** プログラム編集  ***************************************
void	edit_Prg_main(void)								//プログラムエディタメイン
void	set_Prg(byte ln)									//プログラム更新　表示->memへ

//*** センサーモニタ   ************************************
void	do_Setting_main(void)							//センサーモニタ動作メイン
void	moni_AD(void)											//AD入力モニタ
void	moni_Sonar(void)									//超音波センサモニタ
void	moni_Compas(void)									//コンパスモニタ
void	moni_Servo(void)									//サーボ動作テスト
void	moni_Serial(void)									//シリアル通信

//*** シリアル通信            ****************************************
int		chk_Serial()											//シリアル受信チェック
void	rcv_Serial()											//シリアル通信受信
void  clear_Srcv()											//シリアルバッファ初期化
void	com_PC()													//PCと通信

//***   prgデータ操作    **********************************************
void	clear_AllLine(void)								//プログラム全行消去
void	setPrg_clrLine(int y, int x)			//プログラム一行消去
void	setPrg_InsLine(byte l)						//プログラム一行挿入
void	setPrg_DelLine(byte l)						//プログラム一行削除
int		setPrg_seekLast(void)							//プログラム最後の行を返す
void	setPrg_cpyLine(int a, int b)			//プログラム一行コピー(b->a)
void	setPrg_Pattern(int pl)						//コマンドの基本パターンセット

//*** コンパス  *********************************************
void	init_GY271(void)									//コンパスモジュール初期化
void	read_Angle(void)									//コンパス読み込み

//*** prgデータ操作　EEPROM    ***************************************
int		chk_ROM(void)											//ROM書き込みあるかチェック
void	init_ROM(void)										//ROM＋prg初期化
void	read_AllROM(void)									//ROM全読み込み
void	write_AllROM(void)								//ROM全　書込み

//-----         */


#include <EEPROM.h>
#include "NRob.h"
#include "NRobDisp.h"
//#include "NRobDispMode.h"
//#include "NRobDispPrg.h"

//****  定数宣言        ********************************************

#define PICH  10              //割込み　ms

#define PAGENUM   4           //選択ページ枚数

#define MAXLINE   99          //プログラム行数

#define BLANK     0x20        //空白判定　charコード


/*
//  GY271                     //コンパスセンサー
#define ADR271    0x0D        //GY271アドレス
#define OFFSET_X  184  //479       //コンパスオフセット
#define OFFSET_Y  688  //1142

//  GY9250
#define ADR9250   0x68        //MPU9250アドレス
#define ADR8963   0x0C        //磁気センサのスレーブアドレス
#define SCALEACCX  0.0047886  //加速度換算比率
#define SCALEACCY  0.0047886
#define SCALEACCZ  0.0047886

#define SCALEMAGX  0.177467   //コンパス換算比率
#define SCALEMAGY  0.177467
#define SCALEMAGZ  0.171024

#define OFFSET_X  -5.24       //コンパスオフセット
#define OFFSET_Y  +36.74
#define OFFSET_Z  +27.96
*/


//****  グローバル変数  ************************************

NRob3			nrob;													//初期化
NRobDsp		ndsp(nrob);
//NDspMode	dsmd(nrob);
//NDspPrg		dspr(nrob);

//------    プログラム本体変数   ------------------------
byte			prgMem[MAXLINE+1][5];					//プログラムデータ

int				prCnt=0;											//プログラムカウンタ
int				memStack[5];									//スタックメモリ-プログラムカウンタ
byte			memJudg[5];										//判定値スタック
int				cntStack=0;										//スタックポインタ
int				topBlock=0;										//ブロック命令始まる場所
int				prgLast=0;										//ﾌﾟﾛｸﾞﾗﾑ最後

//------    タイマ変数   ------------------------
/*
byte			flg10ms=0;
byte			flg50ms=0;
byte			flg100ms=0;
unsigned long	intTm10;
unsigned long	intTm50;
unsigned long	intMill;
*/
//------    モード切替部変数    ------------------
byte			mode=0;												//モード 0=chg/1=act/2=monitor/3,4=Edit
byte			chgmode=1;

//------    プログラム実行部変数    --------------
//byte		isRemokon=0;									//リモコン使ったプログラムか
byte			isServo=0;										//サーボ命令あるか？
byte			isCompass=0;									//コンパス命令あるか？
//byte			isSonar=0;										//超音波命令あるか？

byte			jdgLvl;												//判定値レベル
byte			jdgWord;											//判定値
byte			prgJdgW;											//プログラムの判定語

int				Tim[3];												//タイマ v361
int				X[10];												//変数
int				dspL[10];											//動作表示  0-2:LED/3-6:LCD(int)
int				calc[6];	//[10];							//計算値保管用　v359->361
//S1とS2で計算　C3:比率0-10/C4:大きい値/C5:小さい値
//S3とS4で計算　C6:比率0-10/C7:大きい値/C8:小さい値


//------    プログラム編集部変数    --------------
int				ds0=0;												//現在表示1行の位置
byte			flgSelect=0;						//選択中のフラグ　0=編集前/1=文字編集/2=行編集

//------    センサーモニタ部変数    --------------
byte			sensorPage=1;									//センサーモニターページ

//------    センサー入力データ部変数    ----------
//int				ad[9];												//AD入力値 + US
//int				adPin[6]={0,1,2,3,6,7};
//int				adCnt=1;

int				angAbs;												//100%換算角度（360°=100）
//int				orgAng;												//スタート時の角度
int				angRel;												//スタート時相対角度　スタート時50

//byte		is9250=0;											//GY9250あり？
//float		mgx, mgy, mgz;								//9軸センサ読込
//float		acx, acy, acz;

//------    通信関係変数    ----------------------
byte			rcvBuf[20];										//シリアル受信バッファ
int				srCnt=-1;											//シリアル受信カウンタ
byte			flgSrcv=0;										//シリアル受信フラグ

//------    LCD表示文字　v361	--------------------
const char titlAct[] PROGMEM = {"Program run"};


//------    プログラムディバッグ用    ----------------------
/* debug
byte			testB;

void	test_prgSir(int ln)		//テストモニタ
{
Serial.print(ln);
Serial.print(": ");
Serial.print(prgMem[ln][0]);
Serial.print(": ");
Serial.print(prgMem[ln][1]);
Serial.print(": ");
Serial.print(prgMem[ln][2]);
Serial.print(": ");
Serial.print(prgMem[ln][3]);
Serial.print(": ");
Serial.println(prgMem[ln][4]);
}
//-- */

//****  メイン *****************************************************
//------------------------------------------------
void	setup()
{
	Serial.begin(9600);
  
	nrob.init(50);												//ハード初期化 ピッチ50ms
	ndsp.dsp_modeStr(0);									//初期モード選択画面

	init_PrgData();												//プログラムデータ初期化と読込

}

//------------------------------------------------
void loop()
{
															//---   モード動作 ---
	switch (mode) {
	 case 0:															//モード変更動作
		chg_Mode_main();
		break;
	 case 1:															//action　動作
		do_Action_main();
		break;
	 case 2:															//sensor  センサーモニタ
		do_Setting_main();
		break;
	 case 3:															//prog  プログラムnew
	 case 4:															//prog  プログラム
		edit_Prg_main();
		break;
	}
}
//------------------------------------------------
void	PrgReset()												//ハードウエアリセット
{
	asm volatile ("  jmp 0");							//　リセット
}

//*** 初期化  ***********************************************
//------------------------------------------------
void	init_PrgData(void)								//プログラムデータ初期化
{
	int	i;

	ds0=1;																//表示用初期行

	mode=0;																//モード 0=chg
	chgmode=1;														//  1=act

	if(chk_ROM()){												//データあれば読込
		read_AllROM();
	}else{																//なければ初期化
		init_ROM();
	}
																				//その他動作関係変数初期化
	prCnt = 1;
	for(i=0; i<3; i++)		Tim[i]=0;				//v361
	for(i=0; i<10; i++){
		dspL[i]=0;
		X[i]=0;
	}
}
//------------------------------------------------

//*** モード変更動作  ***************************************
//------------------------------------------------
void	chg_Mode_main(void)								//モード変更動作メイン
{
	nrob.LCD_noCursor();									//カーソル消す
	nrob.LCD_noBlink();

	chgmode=1;
	ndsp.dsp_modeStr(chgmode);
	while(1){
    nrob.loop();

		if(nrob.in_SW(2)){									//Up
			if(chgmode<PAGENUM) chgmode++;
			else								chgmode=1;
			ndsp.dsp_modeStr(chgmode);
		}
		if(nrob.in_SW(1)){									//Down
			if(chgmode>1) chgmode--;
			else					chgmode=PAGENUM;
			ndsp.dsp_modeStr(chgmode);
    }
		if(nrob.in_SW(0)){									//Sel SW
			mode = chgmode;										//選択ＯＫ
		break;
		}
		if(chk_Serial()){										//シリアル通信あれば v358
			mode = 2;													// PCモニタへジャンプ
			sensorPage = 5;
			break;
		}
	}
	ndsp.dsp_modeStr(0);               //Wait表示で戻る
}


//*** 動作  *************************************************
//------------------------------------------------
void	do_Action_main(void)							//プログラム実行動作メイン
{

	prg_Analyse();												//プログラム解析実行切替
	if(isServo){
		nrob.servo_ON();
	}

	nrob.LCD_clear();											//実行画面表示
	nrob.LCD_setCursor(0, 0);
	nrob.LCD_printF(titlAct);							//v361
	dsp_L_All();													//L変数も表示

	prCnt = 1;

	act_Wait(5);												//500ms待ってスタート

	while(1){
		timcnt_Action();									//動作時基本タイマーと動作
																				//プログラム行順実行
		if(prgMem[prCnt][1]!=0){
			if(ctrl_Branch()){								//条件分岐
				do_command();										//コマンド動作
			}else{
				prCnt++;
			}
			if(cntStack==0){									//メインプログラムの最後
				if(prCnt>=topBlock){
					PrgReset();										//　リセット
				}
			}
		}else{
			if(cntStack>0){										//サブルーチン動作中なら
				do_return();
			}else{														//通常ルーチンならトップへ
				PrgReset(); 										//　エラーリセット
			}
		}
	}

}
//------------------------------------------------
int	timcnt_Action(void)								//動作時基本タイマーと動作
{
	int ans=0;
	float ang;
	static int uno=1;
	static byte flg100ms=0;

	if(nrob.loop()){											//50ms毎動作

		if(isCompass){											//コンパスあれば読込
			ang = (float)nrob.get_Angle();		//絶対値
			angAbs = int((ang*100.0)/360.0);	//360->100 換算
			ang = (float)nrob.get_RelAng();		//相対値
			angRel = int((ang*100.0)/360.0);
		}
		dif_Calc();													//入力差計算 v359
	
		if(flg100ms){												//100ms毎タイマカウント
			flg100ms=0;
			Timer_Count();
			ans=1;
		}else{
			flg100ms=1;
		}
	}
	return ans;

}
//------------------------------------------------
void	dif_Calc(void)										//入力差計算 v359
{
	int	a,b,c;

	a = nrob.get_AD(1)/10;						//S1,S2 -> 0-2 へ
	if(a>99)  a=99;										//0～1024を0-99に換算
	b = nrob.get_AD(2)/10;
	if(b>99)  b=99;

	if(a>b){													//大小振分け
		calc[0]=10-((b*5)/a);
		calc[1]=a;
		calc[2]=b;
	}else{
		calc[0]=(a*5)/b;
		calc[1]=b;
		calc[2]=a;
	}

	a = nrob.get_AD(3)/10;						//S3,S4 -> 3-5 へ
	if(a>99)  a=99;
	b = nrob.get_AD(4)/10;
	if(b>99)  b=99;

	if(a>b){													//大小振分け
		calc[3]=10-((b*5)/a);
		calc[4]=a;
		calc[5]=b;
	}else{
		calc[3]=(a*5)/b;
		calc[4]=b;
		calc[5]=a;
	}
}
//------------------------------------------------
void	act_Wait(int c)										//動作ウエイト　c:待ち時間(x100ms)
{
	while(1){
		if(timcnt_Action()){								//動作時基本タイマーと動作
			if(c--<=0)	break;								//100ms
		}
	}
}
//------------------------------------------------
void	prg_Analyse(void)									//プログラム解析
{
	int i=1;
	byte	isSonar=0;
	byte	isCol=0;												//v350
	byte* buf;

	isServo=0;														//サーボ命令あるか
	isCompass=0;													//コンパス使用しているか？
	topBlock=0;

	while(prgMem[i][1]!=0){								//すべての行の

		buf=ndsp.chg_PrgDsp(0, i, prgMem[i]);		// プログラム文字変換
		
		for(int j=0; j<16; j++){								// 1行内の文字を調べる
			switch(buf[j]){
				case 'S':
					if(isSonar<1){
						if(buf[j+1]=='7')	isSonar=1;	//v355
					}
					if(isSonar<2){									//v360
						if(buf[j+1]=='8')	isSonar=2;
					}
					break;

				case 'C':
					if(buf[j+1]=='1'){						//v359
						isCompass = 1;
					}
					break;

				case 'V':
					isServo = 1;
					break;

				case 'I':														//v350
					isCol = 1;
					break;
			}
		}
		if(buf[2]=='B'){
			if(topBlock==0)	topBlock=i; //ブロック命令のトップをメモリ
		}

		i++;

	}
	nrob.use_US(isSonar);									//超音波設定をnrobにセット
	if(isCol)	nrob.use_SubIn(1);					//v351
	if(topBlock==0)	topBlock=i;
	prgLast=i;

}

//------------------------------------------------
byte	ctrl_Branch(void)									//条件分岐  ans:1=実行/2=スキップ
{
	byte ans=0;

	jdgLvl = (prgMem[prCnt][0]>>4);				//階層レベル
	prgJdgW = prgMem[prCnt][0]&0x0F;

	if(jdgLvl==0){												//ネストなければ未判定
		jdgWord=0;
		ans=1;

	}else{																//ネストあれば判定値比較

		switch (jdgLvl) {
		 case 1:														//
			jdgWord &= 0x01;
			prgJdgW &= 0x01;
			break;
		 case 2:														//
			jdgWord &= 0x03;
			prgJdgW &= 0x03;
			break;
		 case 3:														//
			jdgWord &= 0x07;
			prgJdgW &= 0x07;
		}
		if(jdgWord==prgJdgW)	ans=1;				//階層ごと比較
	}

	return ans;
}

//------------------------------------------------
void	do_command(void)									//コマンド動作
{
	if(prgMem[prCnt][1]<40){
		if(prgMem[prCnt][1]<10)       do_MoveCom();
		else if(prgMem[prCnt][1]<20)  do_IfCom();     //?
		else if(prgMem[prCnt][1]<30)  do_WithinCom(); //$
		else 													do_SetTim();	   //#T   v360
		prCnt++;																			//次の行

	}else if(prgMem[prCnt][1]<50){
		do_JumpCom();   															//JorB
	}else{
		do_SetX();																		//#X   v360
		prCnt++;
	}
}

//------------------------------------------------
void	do_MoveCom(void)									//コマンド 動作系
{

	switch (prgMem[prCnt][1]) {
		case 1:															//Gコマンド
			do_Go();
			break;
		case 2:															//Mコマンド
			do_Motor();
			break;
		case 3:															//Vコマンド
			do_Servo();
			break;
		case 4:															//Wコマンド
			do_Wait();
			break;
		case 5:															//Lコマンド
			do_LEDLCD();
			break;
		case 6:															//Oコマンド v351
			do_SubOut();
			break;
	}
}
//------------------------------------------------
void	do_SubOut(void)										//O subOutボードへ書込み v351
{
  int i,j;
  i = prgMem[prCnt][2];               	//配列NO
  if(i<0 || i>9)  return;
  i--;

  j = chg_Argument(prgMem[prCnt][3]); 	//代入値
	j = (j*255)/100;

	nrob.write_SubOut(i,j);
}

//------------------------------------------------
void	do_Go(void)												//Goコマンド モーター＋wait
{
  int a = chg_Argument(prgMem[prCnt][2]);   //数字変換して
  int b = chg_Argument(prgMem[prCnt][3]);
  int c = chg_Argument(prgMem[prCnt][4]);

  nrob.act_motorA((a*255)/100);      		//ドライバ駆動
  nrob.act_motorB((b*255)/100);
	act_Wait(c);
  nrob.act_motorA(0);               		//停止
  nrob.act_motorB(0);

}
//------------------------------------------------
void	do_Motor(void)										//モーターコマンド
{
  int a = chg_Argument(prgMem[prCnt][2]);   //数字変換して
  int b = chg_Argument(prgMem[prCnt][3]);

  nrob.act_motorA((a*255)/100);      		//ドライバ駆動
  nrob.act_motorB((b*255)/100);
}
//------------------------------------------------
void	do_Servo(void)										//サーボ
{
  int no,pw;

  no=(int)(prgMem[prCnt][2]);						//配列NO
  if(no>0 && no<3){
    pw = chg_Argument(prgMem[prCnt][3]);
  	pw = (pw*9)/5;    //180度換算
  	if(pw<9)   pw=9;
  	if(pw>180) pw=180;
		
    nrob.act_ServoAbs(no, pw);
  }
}
//------------------------------------------------
void	do_Wait(void)											//Waitコマンド
{
  int wt = chg_Argument(prgMem[prCnt][2]);
	act_Wait(wt);
}
//------------------------------------------------
void	do_IfCom(void)										//? 判定動作
{
  int i,j,dt;
  byte  ans=0;

  if(prgMem[prCnt][2]>9)  return;   		//配列以上は何もしない

  i = prgMem[prCnt][2];                 //配列NO
  j = chg_Argument(prgMem[prCnt][4]);   //比判定値

  if(i<0 || i>9)  return;

	dt = get_Variable(prgMem[prCnt][1]-10,i);	//変数

  switch(prgMem[prCnt][3]){         		//記号に合わせて判定

    case 243:                       		//=
      if(dt==j)     ans = 1 ;
      break;
    case 240:                       		//<
      if(dt<j)      ans = 1 ;
      break;
    case 241:                       		//>
      if(dt>j)      ans = 1 ;
      break;
    case 242:                       		//!
      if(dt!=j)      ans = 1 ;
      break;
  
    default:  return;
  }

  switch(jdgLvl){             					//判定結果を値にセット

    case 0:
      jdgWord=0;
      if(ans) jdgWord |= 0x01;
      break;

    case 1:
      jdgWord &= 0x01;
      if(ans) jdgWord |= 0x02;
      break;

    case 2:
      jdgWord &= 0x03;
      if(ans) jdgWord |= 0x04;
      break;

    case 3:
      jdgWord &= 0x07;
      if(ans) jdgWord |= 0x08;
      break;

    default:  return;             			//4階層以上にはならない
  
  }
  jdgLvl++;

}
//------------------------------------------------
void	do_WithinCom(void)								//$ 範囲判定動作
{
  int i,lo,hi,dt;
  byte  ans=0;

  if(prgMem[prCnt][2]>9)  return;   		//配列以上は何もしない

  i = prgMem[prCnt][2];             		//配列NO
  lo = chg_Argument(prgMem[prCnt][3]);
  hi = chg_Argument(prgMem[prCnt][4]);

  if(i<0 || i>9)  return;

	dt = get_Variable(prgMem[prCnt][1]-20,i);	//変数

  if(dt>lo && dt<hi)  ans = 1;      		//範囲判定

  switch(jdgLvl){             					//判定結果を値にセット

    case 0:
      jdgWord=0;
      if(ans) jdgWord |= 0x01;
      break;

    case 1:
      jdgWord &= 0x01;
      if(ans) jdgWord |= 0x02;
      break;

    case 2:
      jdgWord &= 0x03;
      if(ans) jdgWord |= 0x04;
      break;

    case 3:
      jdgWord &= 0x07;
      if(ans) jdgWord |= 0x08;
      break;

    default:  return;             			//4階層以上にはならない
  
  }
  jdgLvl++;
}
//------------------------------------------------
void	do_SetX(void)											//# 変数セット命令 X	v360
{
  int i,j,k;
  i = prgMem[prCnt][1]-50;             	//配列NO
  if(i<0 || i>9)  return;
  j = chg_Argument(prgMem[prCnt][2]); 	//代入値
  k = chg_Argument(prgMem[prCnt][4]);

  switch(prgMem[prCnt][3]){     				//記号に合わせて演算
    case 243: X[i] = j;  	break;  			// =
    case 244: X[i] = j+k; break;  			// +
    case 245: X[i] = j-k; break;  			// -
    case 246: X[i] = j*k; break;  			// *
    case 247: X[i] = j/k; break;  			// /
  }
}
//------------------------------------------------
void	do_SetTim(void)										//# 変数セット命令 T
{
  int i,j;
  i = prgMem[prCnt][2];               	//配列NO
  if(i<0 || i>2)  return;
  j = chg_Argument(prgMem[prCnt][4]); 	//代入値

  switch(prgMem[prCnt][3]){       			//記号に合わせて演算
    case 243: Tim[i] = j;   break;
    case 244: Tim[i] += j;  break;
    case 245: Tim[i] -= j;  break;
    case 246: Tim[i] *= j;  break;
    case 247: Tim[i] /= j;  break;
  }
}
//------------------------------------------------
void	do_LEDLCD(void)										//L LED,LCD操作
{
  int i,j;
  i = prgMem[prCnt][2];               	//配列NO
  if(i<0 || i>9)  return;
  j = chg_Argument(prgMem[prCnt][3]); 	//代入値

  i--;
  dspL[i] = j;

  if(i<3){                          		//表示動作// 123はLED
    if(dspL[i])     nrob.LED_ON(i);
    else            nrob.LED_OFF(i);
  }else if(i<8){                    		// 4-7はLCD
    nrob.dsp_Number2(((i-3)*4), 1, dspL[i]);
  }
}
//------------------------------------------------
void	dsp_L_All(void)										//L変数表示
{
  for(int i=0; i<4; i++)
    nrob.dsp_Number2((i*4), 1, dspL[i]);
}

//************************************************
void	do_JumpCom(void)									//J 動作
{

	switch(prgMem[prCnt][1]){							//番号ごと動作分け	v361

		case 40:do_return();									//Block終わりでreturn
						break;
		case 41:do_jumpB();										//Blockジャンプ(サブルーチン)
						break;
		case 42:do_jumpN();										//番号へ直接ジャンプ
						break;
		case 43:do_loopQ();										//条件付きループ
						break;
		case 44:do_loopB();										//ループ飛び出し
						break;
		case 45:do_loopS();										//無条件ループ開始
						break;
		case 46:do_loopE();										//ループの終わり
						break;

	}
}
//------------------------------------------------
void	do_return(void)										//Block終わりでreturn
{
  if(cntStack==0){          //スタック０ならjikkouエラー
		PrgReset();
  }else{
    cntStack--;         
    prCnt=memStack[cntStack];         //スタックメモリ戻す
    jdgWord=memJudg[cntStack];
  }
	prCnt++;
}
//------------------------------------------------
void	do_jumpB(void)										//Blockジャンプ(サブルーチンジャンプ)
{
  int newCnt;   												//ジャンプ先の番号

  newCnt=seek_Block(prgMem[prCnt][2]);  //ジャンプ先があれば飛ぶ
  if(newCnt){
    if(cntStack<4){                     //スタックは４段まで
      memJudg[cntStack]=jdgWord;        //スタックメモリ
      memStack[cntStack]=prCnt;
      cntStack++;
			jdgWord=0;
      prCnt=newCnt+1;                   //プログラムカウンタ変更
    }
  }else{
		prCnt++;														//ブロックなければ何もしない
	}
}
//------------------------------------------------
int		seek_Block(byte no)								//Block を探す
{
  int i=topBlock;   										//プログラムカウンタ

  while(prgMem[i][1]!=0){
    if(prgMem[i][1]==40){     					//'B'を探す
      if(prgMem[i][2]==no){
        return i;               				//該当番号あれば　行番号を返す
      }
    }
    i++;
  }
  return 0;                   					//なければ0を返す
}
//------------------------------------------------
void	do_jumpN(void)										//番号へ直接ジャンプ
{
  if(prgMem[prCnt][2]>=0){              //プログラムのある行なら
    if(prgMem[prCnt][2]<prgLast){
      prCnt=prgMem[prCnt][2];         //プログラムカウンタ更新
			jdgWord=0;
    }
  }
	if(prCnt<0)	prCnt=0;
}

//------------------------------------------------
void	do_loopQ(void)										//条件付きループ	v361
{
	if(judg_234(prCnt)){									//判定正で
		prCnt++;														//　ループ内へ
	}else{																//判定偽で
		prCnt = seek_loopEnd()+1;						// ループ終わりの次の行に
	}
}
//------------------------------------------------
void	do_loopB(void)										//ループ飛び出し	v361
{
	if(judg_234(prCnt)){									//判定正で
		prCnt = seek_loopEnd()+1;						// ループ終わりの次の行に
	}else{																//判定偽で
		prCnt++;														//　ループ継続
	}
}
//------------------------------------------------
byte	judg_234(int no)									//パラメータ２，３，４の判定	v361
																				//no=プログラムカウンタ
{
  int i,j;
	byte	ans=0;
  i = chg_Argument(prgMem[no][2]);			//引数１
  j = chg_Argument(prgMem[no][4]);			//引数２

  switch(prgMem[no][3]){   		      		//記号に合わせて判定
    case 243: if(i==j)	ans = 1;	break;
    case 240:	if(i<j)		ans = 1;	break;
    case 241:	if(i>j)		ans = 1;	break;
    case 242:	if(i!=j)	ans = 1;	break;
  }

	return ans;
}

//------------------------------------------------
void	do_loopS(void)										//無条件ループ開始	v361
{
	prCnt++;															// この行は何もしない
}
//------------------------------------------------
void	do_loopE(void)										//ループの終わり	v361
{
	prCnt=seek_loopTop();									//ループの先頭に
}
//------------------------------------------------
int	seek_loopTop(void)									//ループの始まりを探す	v361
{
	int i=prCnt-1;
	int	cnt=0;														//２重ループ用カウンタ
	byte	com;

	while(i>1){														//ループ始まりを探す
		com=prgMem[i][1];
		if(com==43 || com==45){							//始まり見つかった
			if(cnt>0){												// 2重状態なら
				cnt--;
			}else{														//そうでないなら
				break;													//　見つかった
			}
		}else if(com==46){									//ループが２重になっている
			cnt++;														// カウンタUpして記録
		}
		i--;
	}
	return i;
}
//------------------------------------------------
int	seek_loopEnd(void)									//ループの終わりを探す	v361
{
	int i=prCnt+1;
	int	cnt=0;														//２重ループ用カウンタ
	byte	com;

	while(i>0){														//ループ始まりを探す
		com=prgMem[i][1];
		if(com==46){												//終わり見つかった
			if(cnt>0){												// 2重状態なら
				cnt--;
			}else{														//そうでないなら
				break;													// 見つかった
			}
		}else if(com==43 || com==45){				//ループが２重になっている
			cnt++;														// カウンタUpして記録
		}
		i++;
	}
	return i;
}

//************************************************

//------------------------------------------------
int		chg_Argument(byte n)							//引数を数字に変換 正負＋変数
{
  int ans;

  if(n<200){                  					//正負値
    if(n<100)   ans = (int)n;
    else        ans = -(int)(n-100);

  }else{
		ans = n - 200;
		ans = get_Variable(ans/10, ans%10);				//変数 v350

  }
  return ans;
}
//------------------------------------------------
int		get_Variable(int m,int n)					//変数変換 v350
{
	int ans;

  switch(m){ 
		case 0:	ans = get_AD(n); break;					//S
    case 1:	ans = get_Compas(n);  break;		//C v359
    case 2:	ans = X[n];  break;							//X
    case 3:																	//T
				if(n<0 || n>2)	ans=0;								//v361
 				else						ans = Tim[n];  
				break;
    case 4: ans = get_COLsenser(n);	break;	//I
    default: ans = 0;
  }
	return ans;

}
//------------------------------------------------
int		get_Compas(byte n)								//コンパス値+計算値 v359
{
	int ans;

	if(n<1){															//C0は乱数 v363
		ans=random(0,99);
	}else if(n<3){												//1はコンパス
		ans=angRel;
	}else if(n<9){												//3-8は計算値
		ans=calc[n-3];
	}else{
		ans=0;
	}
	return ans;
}
//------------------------------------------------
int		get_COLsenser(byte n)							//カラーセンサ値変換
{
	int ans;
	if(n==0)	n=9;
	else			n=n-1;

	ans = nrob.get_SubIn(n);							//v351
	ans /= 10;
	if(ans>99)	ans=99;
	return ans;
}
//------------------------------------------------
void	Timer_Count(void)									//タイマーカウント
{
//  for(int i=0; i<10; i++){	v361
  for(int i=0; i<3; i++){
    Tim[i]++;
    if(Tim[i]>=100) Tim[i]=0;
  }
}

//------------------------------------------------

//*** AD入力    *********************************************
int	get_AD(int an)										//ADを取込 v351 S9 S0　
{
	int ans=0;

	if(an<0){															//エラー
		ans=0;

	}else if(an==0){											//S0はSW状態 v363
		for(int i=0;i<5 ;i++){							//SW調べる
			if(nrob.key_SW(i)){								//押されていれば番号返す
				ans=i+1; break;
			}
		}

	}else if(an<=6){											//A1-A6　アナログ
		ans = nrob.get_AD(an)/10;						//0～1024を0-99に換算
		if(ans>99)  ans=99;									//上限値
		
	}else if(an<=8){											//A7,8 超音波
		ans = nrob.get_US(an-6);
  	if(ans>99)  ans=99;
  	if(ans<1)   ans=99;

	}else if(an==9){											//SW入力
		for(int i=0;i<5 ;i++){								//SW調べる
			if(nrob.in_SW(i)){									//押されていれば番号返す
				ans=i+1; break;
			}
		}
	}else{																//エラー
		ans=0;
	}
	return ans;
}
//------------------------------------------------

//*** プログラム編集  ***************************************
//------------------------------------------------
void	edit_Prg_main(void)								//プログラムエディタメイン
{
	byte	ans;

	ds0=1;																//1行目から表示
	flgSelect=0;													//非選択状態で開始

	if(mode==3){
		clear_AllLine();										//プログラム新規メモリクリア
	}
	ndsp.reDraw(ds0, prgMem[ds0]);				//初期表示
	ndsp.cursor_Home();

	while(1){
		nrob.loop();
		//------              編集動作
		if(flgSelect){											//選択動作中
			if(flgSelect==2){
				ans = ndsp.edit_InsDel();				//行編集
				if(ans){
					if(ans>1){										//挿入削除確定
						if(ans>3)	setPrg_InsLine(ans-3);
						else			setPrg_DelLine(ans-1);
					}
					flgSelect=0;
					ndsp.reDraw(ds0, prgMem[ds0]);//表示戻す
					ndsp.adjust_curX();
				}
			}else{
				ans=ndsp.edit_Chging();					//プログラム更新中
				if(ans){
					if(ans>10){										//コマンドの更新
						ans-=10;
						set_Prg(ans);								//表示prg変換し
						setPrg_Pattern(ds0+ans-1);	//コマンドパターンをセット

					}else{
						if(ans<9){									//その他の更新
							set_Prg(ans);
						}
					}
					flgSelect=0;
					ndsp.reDraw(ds0, prgMem[ds0]);	//表示戻す
					ndsp.adjust_curX();
				}
			}
		}else{															//選択前　移動動作

			ans=ndsp.edit_Moving();						//移動中選択キーで選択動作
			switch(ans){											//表示変更あれば
			 case 1:
				flgSelect=1;										//選択動作へ
				nrob.LCD_blink();								//ブリンク
				break;
			 case 2:
				flgSelect=2;										//挿入削除動作に
				break;
			 case 3:
				if(ds0>1){											//スクロールアップ
					ds0--;
					ndsp.reDraw(ds0, prgMem[ds0]);//再表示
					ndsp.adjust_curX();
				}
				break;
			 case 4:
				if(ds0<MAXLINE){								//スクロールダウン　最下行まで可
					ds0++;
					ndsp.reDraw(ds0, prgMem[ds0]);//再表示
					ndsp.adjust_curX();
				}
			}
			if(nrob.in_LongSW(1)){						//v356スクロールspeedup
				ds0+=10;
				if(ds0>MAXLINE)	ds0=MAXLINE-1;
				ndsp.reDraw(ds0, prgMem[ds0]);//再表示
				ndsp.adjust_curX();
			}
			if(nrob.in_LongSW(2)){						//v356スクロールspeedup
				ds0-=10;
//				if(ds0<0)	ds0=0;
				if(ds0<1)	ds0=1;								//行１まで v363 バグ修正
				ndsp.reDraw(ds0, prgMem[ds0]);//再表示
				ndsp.adjust_curX();
			}
    }

		if(nrob.in_LongSW(0)){							//Sel SW 長押し メモリ
			ndsp.dsp_modeStr(0);							//Wait表示
			write_AllROM();										//プログラム保存
			mode = 0;													//トップ画面に
			break;
		}
	}
}
//------------------------------------------------
byte	set_Prg(byte ln)									//プログラム更新　表示->memへ
{
	byte* buf=ndsp.chg_DspPrg(ln);				//表示データから変換

	prgMem[buf[0]][0]=buf[1];							//プログラムデータへ
	prgMem[buf[0]][1]=buf[2];
	prgMem[buf[0]][2]=buf[3];
	prgMem[buf[0]][3]=buf[4];
	prgMem[buf[0]][4]=buf[5];

}


//*** センサーモニタ   ************************************
//------------------------------------------------
void	do_Setting_main(void)							//センサーモニタ動作メイン
{
//	sensorPage=1;			v358

  while(1){

    switch (sensorPage){
      case 1:                     			//AD入力モニタ
        moni_AD();      
        break;
      case 2:                    				//超音波センサモニタ
        moni_Sonar();
        break;
      case 3:                     			//コンパスモニタ
        moni_Compas();
        break;
      case 4:                     			//サーボ動作テスト
        moni_Servo();
        break;
      case 5:                     			//シリアル通信
        moni_Serial();
        break;
    }
  }
	sensorPage=1;

}
//------------------------------------------------
void	moni_AD(void)											//AD入力モニタ
{
	ndsp.moni_AD_set();										//ベース表示

  while(1){
		nrob.loop();
		ndsp.moni_AD_loop();								//AD読込と表示

//		timer_getSW();											//動作ピッチとSW読込
	  if(nrob.in_SW(0)){           				//selSW IN?
      sensorPage++;
      break;
		}
  }
}
//------------------------------------------------
void	moni_Sonar(void)									//超音波センサモニタ
{
	ndsp.moni_Sonar_set();
	nrob.use_US(2);

  while(1){

		if(nrob.loop()){

			ndsp.moni_Sonar_loop();						//測定表示
    }

//		timer_getSW();											//動作ピッチとSW読込
	  if(nrob.in_SW(0)){           				//selSW IN?
      sensorPage++;
      break;
		}
  }
	nrob.use_US(2);
}
//------------------------------------------------
void	moni_Compas(void)									//コンパスモニタ
{
	float ang;

	ndsp.moni_Compas_set();

  while(1){

		if(nrob.loop()){

			ang = (float)nrob.get_Angle();		//絶対値
			angAbs = int((ang*100.0)/360.0);	//360->100 換算
			ang = (float)nrob.get_RelAng();		//相対値
			angRel = int((ang*100.0)/360.0);

      nrob.dsp_Number2(10, 0, angAbs);
      nrob.dsp_Number2(10, 1, angRel);
    }

	  if(nrob.in_SW(2)){           				//UpSW In?	v351
			nrob.get_OffsetCompas();					//オフセット得る
		}
	  if(nrob.in_SW(0)){           				//selSW IN?
      sensorPage++;
      break;
		}
  }
}
//------------------------------------------------
void	moni_Servo(void)									//サーボ動作テスト
{
  nrob.servo_ON();
  isServo=1;
	ndsp.moni_Servo_set();								//モニタ画面セット

  while(1){
		nrob.loop();

		ndsp.moni_Servo_loop();							//キーに合わせて動作表示

    if(nrob.in_SW(0)){           				//selSW IN?
      sensorPage++;
      break;
    }
  }
  nrob.servo_OFF();
  isServo=0;

}
//------------------------------------------------
void	moni_Serial(void)									//シリアル通信
{
  clear_Srcv();

	ndsp.moni_Serial_set();								//初期表示

  while(1){
		nrob.loop();
		if(nrob.in_SW(0)){									//selSW IN?
      sensorPage=1;											//ページ替え
      break;
    }
                 //--- シリアル受信処理  --- 
    rcv_Serial();               				//受信待ち
    if(flgSrcv){
      com_PC();                 				//モード変更中のみPCと通信
      flgSrcv=0;                				// 次の受信可
    }

  }
  Serial.end();
}


//*** シリアル通信            ****************************************
//------------------------------------------------
int	chk_Serial()												//シリアル受信チェック v358
{
	if(Serial.available() > 0){
		return 1;
	}else{
		return 0;
	}
}

//------------------------------------------------
void	rcv_Serial()											//シリアル通信受信
{
	if(Serial.available() > 0){
		byte c = Serial.read();							//一文字受信

		if(srCnt<0){												//受信待ちの時
			if(c=='['){												// スタートのみ受付
				srCnt=0;
			}
		}else{															//受信中はデータメモリ
			if(c==']'){												// 受信完了でフラグセット
				flgSrcv=1;
				srCnt=-1;
			}else{
				rcvBuf[srCnt] = c;							// データセットして次を待つ
				if(++srCnt>18){									// オーバフロー処理
					clear_Srcv();
				}
      }
    }
  }
}

//------------------------------------------------
void  clear_Srcv()											//シリアルバッファ初期化
{
	for(int i=0; i<18; i++) 
		rcvBuf[i]=BLANK;
	srCnt=-1;
}

//------------------------------------------------
void	com_PC()													//PCと通信
{
	int lno;
	byte* buf;

	switch(rcvBuf[0]){	//受信データ一文字目がコマンド

	 case 'a':														//通信確認
		Serial.println("NRob");
		break;

	 case 'r':														//プログラム読込 ->PC
		lno=((int)(rcvBuf[1]-0x30)*10)			// 番号 数字に変換
				+ (int)(rcvBuf[2]-0x30);

		if(lno>0 && lno<MAXLINE){						// 対象行送信
			if(prgMem[lno][1]!=0){						// プログラムあれば返信
				buf=ndsp.chg_PrgDsp(0, lno, prgMem[lno]);	// プログラム文字変換

				for(int i=0; i<16; i++)					// 1行送信
					Serial.write(buf[i]);
				Serial.write('\n');
			}else{														//なければ終了記号
				Serial.write(rcvBuf[1]);
				Serial.write(rcvBuf[2]);
				Serial.println("[n]");
			}
		}
		break;

	 case 'w':														//プログラム書換え <-PC
		lno=((int)(rcvBuf[1]-0x30)*10)			// 番号 数字に変換
				+ (int)(rcvBuf[2]-0x30);

		Serial.write(rcvBuf[1]);						//書き込み行番号を返す　v358
		Serial.write(rcvBuf[2]);
		Serial.println("[n]");

		if(lno>0 && lno<MAXLINE){
			ndsp.set_prgDsp(0, rcvBuf);				// 表示データに一時メモリ
			set_Prg(0);												// 表示データを変換してプログラムに
		}
		break;

	 case 'e':														//プログラムメモリ
		nrob.LCD_clear();
		write_AllROM();											// ROMへ書き込み
		PrgReset();													//　リセット
		break;
	}
	clear_Srcv();
	delay(10);
}


//***   prgデータ操作    **********************************************
//------------------------------------------------
void	clear_AllLine(void)								//プログラム全行消去
{
	for(int j=0; j<MAXLINE; j++)
		setPrg_clrLine(j,0);								//全データクリア
}
//------------------------------------------------
void	setPrg_clrLine(int y, int x)			//プログラム一行消去
{
	for(int i=x; i<5; i++)
		prgMem[y][i]=0;											// prgクリア
}
//------------------------------------------------
void	setPrg_InsLine(byte l)						//プログラム一行挿入
{
	int nl=ds0+(l-1);											//編集対象行
	int i=setPrg_seekLast();							//プログラム最終行
  
	while(i!=nl){													//最終行から順に一つ送り空き行を作る
		setPrg_cpyLine(i,i-1);
		i--;
	}
	setPrg_clrLine(nl, 0);								//空けた行を初期化
}
//------------------------------------------------
void	setPrg_DelLine(byte l)						//プログラム一行削除
{
	int i=ds0+(l-1);											//編集対象行
  int ll=setPrg_seekLast();							//プログラム最終行
  
	while(i!=ll){													//対象行から順に前に詰める
		setPrg_cpyLine(i,i+1);
		i++;
	}
	setPrg_clrLine(ll, 0);								//最終行をクリア
}
//------------------------------------------------
int		setPrg_seekLast(void)							//プログラム最後の行を返す
{
	int i=1;
	while(i<MAXLINE){
		if(prgMem[i][1]==0){								//コマンド無しならラスト
			break;
		}
		i++;
	}
	return i;
}
//------------------------------------------------
void	setPrg_cpyLine(int a, int b)			//プログラム一行コピー(b->a)
{
	for(int i=0; i<5; i++)
		prgMem[a][i]=prgMem[b][i];					//１行(5byte)コピー
}
//------------------------------------------------
void	setPrg_Pattern(int pl)						//コマンドの基本パターンセット
{
	if(prgMem[pl][1]<10){
		switch (prgMem[pl][1]) {
			case 1:														//Gコマンド
				prgMem[pl][2] = 30;							//G+30+30;10
				prgMem[pl][3] = 30;
				prgMem[pl][4] = 10;
				break;
			case 2:														//Mコマンド
				prgMem[pl][2] = 30;							//M+30+30
				prgMem[pl][3] = 30;
				prgMem[pl][4] = 0;
				break;
			case 3:														//Vコマンド
				prgMem[pl][2] = 1;							//V1;50
				prgMem[pl][3] = 50;
				prgMem[pl][4] = 0;
				break;
			case 4:														//Wコマンド
				prgMem[pl][2] = 10;							//W;10
				prgMem[pl][3] = 0;
				prgMem[pl][4] = 0;
				break;
			case 5:														//Lコマンド
				prgMem[pl][2] = 1;							//L1:01
				prgMem[pl][3] = 1;
				prgMem[pl][4] = 0;
				break;
			case 6:														//Oコマンド
				prgMem[pl][2] = 1;							//O1:30
				prgMem[pl][3] = 30;
				prgMem[pl][4] = 0;
				break;

		}
	}else if(prgMem[pl][1]<20){						//?コマンド
				prgMem[pl][1] = 10;							//?S1<+50
				prgMem[pl][2] = 1;
				prgMem[pl][3] = 241;          
				prgMem[pl][4] = 50;

	}else if(prgMem[pl][1]<30){						//$コマンド
				prgMem[pl][1] = 21;							//$C1;00,50
				prgMem[pl][2] = 1;
				prgMem[pl][3] = 0;
				prgMem[pl][4] = 50;

	}else if(prgMem[pl][1]<40){						//#Tコマンド
				prgMem[pl][1] = 33;							//#T1=+10
				prgMem[pl][2] = 1;
				prgMem[pl][3] = 243;
				prgMem[pl][4] = 10;

	}else if(prgMem[pl][1]<43){						//B,Jコマンド

		if(prgMem[pl][1]==40){
			prgMem[pl][0] = 0;								//B;1
		}else{															//JB:01 v361
//			prgMem[pl][1] = 41;								
		}
		prgMem[pl][2] = 1;
		prgMem[pl][3] = 0;
		prgMem[pl][4] = 0;

	}else if(prgMem[pl][1]<50){						//JLコマンド v361

		if(prgMem[pl][1]<45){								//JLQ:X1=01
			prgMem[pl][2] = 221;
			prgMem[pl][3] = 243;
			prgMem[pl][4] = 1;
		}else{															//JLE:
			prgMem[pl][2] = 0;
			prgMem[pl][3] = 0;
			prgMem[pl][4] = 0;
		}

	}else if(prgMem[pl][1]<60){						//Xコマンド v360

		prgMem[pl][1] = prgMem[pl][1];	//X1=10+0
		prgMem[pl][2] = 10;
		prgMem[pl][3] = 244;
		prgMem[pl][4] = 0;
	}

}

//*** prgデータ操作　EEPROM    ***************************************
//------------------------------------------------
int		chk_ROM(void)											//ROM書き込みあるかチェック
{
	char  w[16];

	for(int i=0; i<7; i++)  w[i] = (char)EEPROM.read(i);
	w[7] = '\0';

	if(w[0]=='D' && w[1]=='A' && w[2]=='T')
		return 1;
	else
		return 0;
}
//------------------------------------------------
void	init_ROM(void)										//ROM＋prg初期化
{
  clear_AllLine();											//全データクリア
  
  EEPROM.write(0, 'D');
  EEPROM.write(1, 'A');
  EEPROM.write(2, 'T');

  write_AllROM();
}

//------------------------------------------------
void	read_AllROM(void)									//ROM全読み込み
{
  int i,j,adr;

  for(j=0; j<MAXLINE; j++){
    for(i=0; i<5; i++){
      adr = j*5 + i + 5;
      prgMem[j][i] = EEPROM.read(adr);
    }
  }
}

//------------------------------------------------
void	write_AllROM(void)								//ROM全　書込み
{
	int i,j,adr;
	int flg=0;

	for(int j=1; j<MAXLINE; j++){					//とび行トリム

		if(flg){
			setPrg_clrLine(j,0);							//とび行以降は空白に
		}else{
			if(prgMem[j][1]==0)	flg=1;				//先頭空白の行は移行はデータはなし
		}
	}

	for(j=0; j<MAXLINE; j++){							//書込み　全行
		for(i=0; i<5; i++){
			adr = j*5 + i + 5;
			EEPROM.write(adr, prgMem[j][i]);
		}
	}
}



